import { ComponentPortal } from '@angular/cdk/portal';
import { Injector, InjectionToken } from '@angular/core';

class NoSuitableRegistration extends Error {
    constructor(m) {
        super(m);
        Object.setPrototypeOf(this, NoSuitableRegistration.prototype);
    }
}

class ComponentRegistry {
    constructor(registrations) {
        this.registrations = registrations;
    }
    find(identifier) {
        const registration = this.registrations.find((it) => it.matches(identifier));
        if (!registration) {
            throw new NoSuitableRegistration(`No such registration found for identifier '${identifier}'`);
        }
        return registration;
    }
}

class InvalidOperation extends Error {
    constructor(m) {
        super(m);
        Object.setPrototypeOf(this, InvalidOperation.prototype);
    }
}

class PortalBuilder {
    constructor(registry, identifier, onPick, injector) {
        this.registry = registry;
        this.identifier = identifier;
        this.onPick = onPick;
        this.injector = injector;
    }
    useRegistry(registryOrRegistrations) {
        const registry = registryOrRegistrations instanceof ComponentRegistry
            ? registryOrRegistrations
            : new ComponentRegistry(registryOrRegistrations);
        return new PortalBuilder(registry, this.identifier, this.onPick, this.injector);
    }
    /**
     * @alias useIdentifier
     */
    pick(identifier) {
        return this.useIdentifier(identifier);
    }
    useIdentifier(identifier) {
        return new PortalBuilder(this.registry, identifier, this.onPick, this.injector);
    }
    whenPicked(onPick) {
        return new PortalBuilder(this.registry, this.identifier, onPick, this.injector);
    }
    useInjector(injectorOrProviders, parent, name) {
        const injector = Array.isArray(injectorOrProviders)
            ? Injector.create({
                providers: injectorOrProviders,
                parent,
                name,
            })
            : injectorOrProviders;
        return new PortalBuilder(this.registry, this.identifier, this.onPick, injector);
    }
    attachTo(outlet) {
        if (!this.registry) {
            throw new InvalidOperation(`No registry or registrations provided. Did you forget to useRegistry()?`);
        }
        if (!this.identifier) {
            throw new InvalidOperation(`No identifier provided. Did you forget to pick() or useIdentifier()?`);
        }
        const registration = this.registry.find(this.identifier);
        if (typeof this.onPick === 'function') {
            this.onPick(registration);
        }
        outlet.detach();
        return outlet.attachComponentPortal(new ComponentPortal(registration.componentType, null, this.injector));
    }
}

const COMPONENT_REGISTRATION = new InjectionToken('ComponentRegistration');

/*
 * Public API Surface of angular-portal-builder
 */

/**
 * Generated bundle index. Do not edit.
 */

export { COMPONENT_REGISTRATION, ComponentRegistry, InvalidOperation, NoSuitableRegistration, PortalBuilder };
//# sourceMappingURL=angular-portal-builder.js.map
